using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using System.Xml.Linq;


namespace DAWalkerProg {
    static class DAWalker {
        [STAThread]
        static void Main() {
            if(System.Diagnostics.Process.GetProcessesByName("DAWalker").Length == 2) return;
            AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(MyUnhandledEx);
            Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(MyThreadingEx);

            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new DAWalker_form());
        }

        static void MyUnhandledEx(object sender, UnhandledExceptionEventArgs args) {
            using(StreamWriter sw = new StreamWriter("CrashLogUnhandledEx (DAWalker " + Application.ProductVersion + ") " + DateTime.Now.ToString("u", System.Globalization.CultureInfo.InvariantCulture).TrimEnd('Z').Replace(":", "-") + ".txt", true)) sw.WriteLine(((Exception)args.ExceptionObject).ToString());
            Environment.Exit(0);
        }

        static void MyThreadingEx(object sender, ThreadExceptionEventArgs args) {
            using(StreamWriter sw = new StreamWriter("CrashLogThreadingEx (DAWalker " + Application.ProductVersion + ") " + DateTime.Now.ToString("u", System.Globalization.CultureInfo.InvariantCulture).TrimEnd('Z').Replace(":", "-") + ".txt", true)) sw.WriteLine(args.Exception.ToString());
            Environment.Exit(0);
        }
    }

    public class DAWalkerStart {
        public struct NodeMap {
            List<int[][]> info;
            public int[] mapAxis;
            public int loadMap;

            public NodeMap(int[] mapAxis1, int loadMap1) {
                mapAxis = mapAxis1;
                loadMap = loadMap1;

                info = new List<int[][]>();
            }
            public void addExit(int mapName, int[] mapExit, int[] mouseLoc) {
                info.Add(new int[][] { new int[] { mapName }, mapExit, mouseLoc });
            }
            public int[] getExitLoc(int toMap) {
                foreach(int[][] A in info) if(A[0][0] == toMap) return A[1];
                return null;
            }
            public int[] getMouseLoc(int toMap) {
                foreach(int[][] A in info) if(A[0][0] == toMap) return A[2];
                return null;
            }
        }


        public SortedList<int, NodeMap> mapNodes = new SortedList<int, NodeMap>();
        private DijkstraImpl myDijkstra = new DijkstraImpl();

        public DAMover.Mover MainMover;
        public Thread MainMoverT;
        public DAMac.DAMacros MainSpecial;
        public DAWalker_form formParent;

        public int curMap, finalMap = 0;
        public int[] path;
        public string pathDA;

        public int spellLoc, optionMain;

        public DAWalkerStart(DAWalker_form parent) {
            formParent = parent;
            MainMover = new DAMover.Mover(this);

            MainSpecial = new DAMac.DAMacros(formParent.Handle);
            MainSpecial.MainF = formParent;
        }

        public void initilizeMap() {
            StreamReader loadFile;
            try { loadFile = File.OpenText("WorldMap.dat"); } catch(Exception e) {
                new Thread(NotificationWindow).Start("Couldn't Load File" + e.Message);
                return;
            }

            pathDA = XElement.Load("DAData.xml").Element("Data").Element("MapFolder").Value;
            if(!Directory.Exists(pathDA)) {
                if(Directory.Exists(@"C:\Program Files\KRU\Dark Ages\maps\")) pathDA = @"C:\Program Files\KRU\Dark Ages\maps\";
                else if(Directory.Exists(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData) + @"\VirtualStore\Program Files (x86)\KRU\Dark Ages\maps\")) pathDA = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData) + @"\VirtualStore\Program Files (x86)\KRU\Dark Ages\maps\";
                else if(Directory.Exists(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData) + @"\VirtualStore\Program Files\KRU\Dark Ages\maps\")) pathDA = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData) + @"\VirtualStore\Program Files\KRU\Dark Ages\maps\";
                else MessageBox.Show("Map Directory Doesn't Exsist; Set the Right Directory in the WorldMap.dat");
            }

            string[] temp;
            List<int[]> arcs = new List<int[]>();
            try {
                while(!loadFile.EndOfStream) {
                    temp = loadFile.ReadLine().Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                    string nameS = "";
                    int name = 0;
                    if(temp[0] != "-1") {
                        int axisx = int.Parse(temp[1]);
                        nameS = temp[0];
                        name = int.Parse(temp[0]);

                        if(axisx == -1) mapNodes[name] = new NodeMap(new int[] { axisx }, 0);
                        else {
                            int axisy = int.Parse(temp[2]);
                            mapNodes[name] = new NodeMap(new int[] { axisx, axisy }, 0);
                        }
                    } else {
                        int axisx = int.Parse(temp[2]);
                        nameS = temp[1];
                        name = int.Parse(temp[1]);

                        if(axisx == -1) mapNodes[name] = new NodeMap(new int[] { axisx }, 0);
                        else {
                            int axisy = int.Parse(temp[3]);
                            mapNodes[name] = new NodeMap(new int[] { axisx, axisy }, 0);
                        }

                        string otherName = "";
                        if(temp.Length > 4) for(int i = 4; i < temp.Length; i++) otherName = otherName + temp[i] + " ";
                        otherName = otherName + "= " + nameS;
                        formParent.otherList.Add(otherName);
                    }

                    if(!loadFile.EndOfStream) temp = loadFile.ReadLine().Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                    while(!loadFile.EndOfStream && temp.Length != 0) {
                        string toName = temp[0];
                        if(toName != "-1") {
                            // add the nodes to the graph, if needed
                            if(!myDijkstra.SoCalMap.Contains(nameS)) myDijkstra.SoCalMap.AddNode(nameS, null);
                            if(!myDijkstra.SoCalMap.Contains(toName)) myDijkstra.SoCalMap.AddNode(toName, null);
                            if((nameS == "502" && toName == "3014") || (nameS == "3014" && toName == "502")) myDijkstra.SoCalMap.AddDirectedEdge(nameS, toName, 10);
                            else myDijkstra.SoCalMap.AddDirectedEdge(nameS, toName, 1);
                        }

                        int[] mouseTemp = new int[temp.Length - 1];
                        for(int i = 1; i < temp.Length; i++) mouseTemp[i - 1] = int.Parse(temp[i]);

                        temp = loadFile.ReadLine().Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);

                        int[] exitTemp = new int[temp.Length];
                        for(int i = 0; i < temp.Length; i++) exitTemp[i] = int.Parse(temp[i]);

                        mapNodes[name].addExit(int.Parse(toName), exitTemp, mouseTemp);
                        if(!loadFile.EndOfStream) temp = loadFile.ReadLine().Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                    }
                }
                loadFile.Close();
            } catch(Exception e) {
                MessageBox.Show("You have a corrupted WorldMap.dat--fix or redownload.\n\nExact error: " + e.Message);
                Environment.Exit(1);
            }
        }

        public void NotificationWindow(object Msg) { MessageBox.Show(Msg.ToString()); }

        //Place = 1(Blossom),2(ToAndor),3(Andor),4(Shinewood),5(Giragan),6(Chaos),7(Cthonic),8(Crypt),9(Suomi),10(Nobis),other(Other)
        //finish = 1(First) or 2(Second)
        //option(3,6-4,7,8) = 0(none),1(HideOn),2(Log),3(Scroll),4(Dach),5(DionOn)
        //spellr = 1,2,3
        //spellc = 1 --> 12
        //item = 1 --> 12
        public int startMac(int place, int finish, int option, int item) {
            MainMoverT = new Thread(new ParameterizedThreadStart(MainMover.startMovement));
            if(!MainMover.StartProcessCheck()) return 1;
            MainMover.UpdateValues();
            curMap = MainMover.MapNum;
            switch(place) {
                case 1:
                    finalMap = 2901;
                    break;
                case 2:
                    finalMap = 10101;
                    break;
                case 3:
                    finalMap = 10180;
                    break;
                case 4:
                    if(finish == 1) finalMap = 559;
                    else finalMap = 1007;
                    break;
                case 5:
                    if(finish == 1) finalMap = 2089;
                    else if(finish == 3) finalMap = 2099;
                    else finalMap = 2096;
                    break;
                case 6:
                    if(finish == 2) finalMap = 3611;
                    else if(finish == 3) finalMap = 4335;
                    else finalMap = 3634;
                    break;
                case 7:
                    if(finish == 1) finalMap = 5001;
                    else if(finish == 2) finalMap = 5010;
                    else finalMap = 5031;
                    break;
                case 8:
                    finalMap = 0;
                    break;
                case 9:
                    finalMap = 503;
                    break;
                case 10:
                    finalMap = 6700;
                    break;
                case 11:
                    if(finish == 1) finalMap = 3938;
                    else if(finish == 2) finalMap = 3935;
                    else finalMap = 3940;
                    break;
                case 12:
                    if(finish == 1) finalMap = 10054;
                    else if(finish == 2) finalMap = 10996;
                    else finalMap = 6998;
                    break;
                default:
                    finalMap = place;
                    if(!mapNodes.ContainsKey(finalMap)) {
                        new Thread(NotificationWindow).Start("Sorry, your destination Map # is not in the Mapped World.  Please choose a different Map Destination.");
                        return 1;
                    }
                    break;
            }
            if(!mapNodes.ContainsKey(curMap)) {
                new Thread(NotificationWindow).Start("Sorry, your current Map # is not in the Mapped World.  Please go to a start location listed in the program.");
                return 1;
            }
            int[] followPath = myDijkstra.ShortestPath(curMap.ToString(), finalMap.ToString());
            if(followPath != null) MainMoverT.Start(followPath);
            else {
                new Thread(NotificationWindow).Start("Path was not found in the World Map.  Please add more path data to World Map.");
                return 1;
            }

            byte startMacro = 0;
            switch(option) {
                case 1:
                    //Keep Hide On
                    startMacro = 1;
                    break;
                case 2:
                    //Log if Hide Runs
                    optionMain = option;
                    startMacro = 2;
                    break;
                case 3:
                    //Scroll if Hide Runs
                    optionMain = option;
                    spellLoc = item;
                    startMacro = 2;
                    break;
                case 4:
                    //Dach if Hide Runs
                    optionMain = option;
                    startMacro = 2;
                    break;
                case 5:
                    //Keep Dion On
                    startMacro = 0;
                    break;
            }
            if(option != 0) MainSpecial.StartHK(startMacro);
            return 0;
        }

        public int startAite() {
            MainMoverT = new Thread(new ParameterizedThreadStart(MainMover.startMovement));
            if(!MainMover.StartProcessCheck()) return 1;
            MainMover.UpdateValues();
            curMap = MainMover.MapNum;

            if(!mapNodes.ContainsKey(curMap)) {
                new Thread(NotificationWindow).Start("Sorry, your current Map # is not in the Maped World.  Please go to a start location listed in the program.");
                return 1;
            }
            int[] followPath = myDijkstra.ShortestPath(curMap.ToString(), "378");
            if(followPath != null) MainMoverT.Start(followPath);
            else {
                new Thread(NotificationWindow).Start("Path was not found in the World Map.  Please add more path data to World Map.");
                return 1;
            }
            return 0;
        }

        public void stopMac() {
            MainMoverT.Abort();
        }

        public void removeCaravan() {
            myDijkstra.SoCalMap.AddDirectedEdge("10000", "10999", 1000);
            myDijkstra.SoCalMap.AddDirectedEdge("10999", "10000", 1000);
        }

        public void addCaravan() {
            myDijkstra.SoCalMap.AddDirectedEdge("10000", "10999", 1);
            myDijkstra.SoCalMap.AddDirectedEdge("10999", "10000", 1);
        }
    }
}